#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function

# We'll need numpy and matplotlib for plotting our results
import numpy as np
import matplotlib.pyplot as plt

# A least squares fitting algorithm from scipy
from scipy.optimize.minpack import leastsq

# DiffPy-CMI modules for building a fitting recipe
from diffpy.Structure import loadStructure
from pyobjcryst import loadCrystal

from diffpy.srfit.pdf import PDFContribution,PDFGenerator, PDFParser
from diffpy.srfit.fitbase import FitRecipe, FitResults,FitContribution,Profile
from ase.io import read, write
from ase.calculators.emt import EMT
from ase.calculators.calculator import Calculator, all_changes
from ase.constraints import constrained_indices
from ase.visualize.plot import plot_atoms
import os
from mpi4py import MPI
class RDF(Calculator):
	"""
	RDF
	"""
	implemented_properties = ['forces', 'energy']

	def __init__(self, Gr, pdfrange,delta=0.1,scale=0.04,qdamp=0.03,psize=None,calcforces=False,periodic=True,label=None):
		Calculator.__init__(self)
		self.Gr = Gr
		self.pdfrange = pdfrange
		self.delta = delta
		self.psize = psize
		self.calcforces=calcforces
		self.scale=scale
		self.qdamp=qdamp
		self.periodic=periodic
		self.comm = MPI.COMM_WORLD
		self.rank = self.comm.Get_rank()
		self.size = MPI.COMM_WORLD.Get_size()
		if label==None:
			self.label='rdftmpfile'
		else:
			self.label=label
#	def rmse(predictions, targets):
#		return np.sqrt(((predictions - targets) ** 2).mean())

	def calc_energy(self,atoms):
#		ff=io.StringIO()
#		rand=str(np.random.randint(10000))
		rand=str(self.rank)
#		print(rand)
		structureFile = self.label+rand+'.cif'
		write(structureFile,atoms,format='cif',parallel=False)
#		write(ff,atoms,format='cif')
		dataFile = self.Gr
#		structureFile = ff
		xmi = self.pdfrange[0]
		xma = self.pdfrange[1]
		niPDF = PDFContribution("G")
		niPDF.loadData(dataFile)
		niPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
		niStructure = loadCrystal(structureFile)
#		niStructure = loadStructure(structureFile,fmt='cif')
		niPDF.addStructure("G", niStructure,periodic=self.periodic)
		if self.psize != None:
			from diffpy.srfit.pdf.characteristicfunctions import sphericalCF
			niPDF.registerFunction(sphericalCF, name = "f")
			niPDF.setEquation("f * G")
		niFit = FitRecipe()
		niFit.addContribution(niPDF)
		niFit.addVar(niPDF.scale, self.scale)
		if self.psize != None:
			niFit.addVar(niPDF.psize, self.psize)
#		niFit.addVar(niPDF.psize, 100)
#		niFit.addVar(niPDF.delta1,2)
		niFit.addVar(niPDF.qdamp, self.qdamp, fixed=True)
		niFit.clearFitHooks()
		if self.optparams:
			leastsq(niFit.residual, niFit.values)
		niResults = FitResults(niFit)
		res=str(niResults).strip().split()
		if self.optparams==False:
			for i in range(len(res)):
        			if res[i]=='Rw':
                			rw=float(res[i+1])
                			break
		else:
			for i in range(len(res)):
        			if res[i]=='Rw':
                			rw=float(res[i+1])
        			elif res[i]=='scale':
                			self.scale=float(res[i+1])
        			elif res[i]=='qdamp':
                			self.qdamp=float(res[i+1])
		energy=rw*100-100
#		try:
#			os.remove(structureFile)
#		except:
#			print("Can not delete the file as it doesn't exists")
#		if os.path.exists(structureFile):
#			os.remove(structureFile)
#		else:
#			print("Can not delete the file as it doesn't exists")
		"""
		r = niFit.nickel.profile.x
		gobs = niFit.nickel.profile.y

# Get the calculated PDF and compute the difference between the calculated and
# measured PDF
		gcalc = niFit.nickel.evaluate()
		baseline = 1.1 * gobs.min()
		gdiff = gobs - gcalc

# Plot!
		plt.figure()
		plt.plot(r, gobs, 'bo', label="G(r) data",
         		markerfacecolor='none', markeredgecolor='b')
		plt.plot(r, gcalc, 'r-', label="G(r) fit")
		plt.plot(r, gdiff + baseline, 'g-', label="G(r) diff")
		plt.plot(r, np.zeros_like(r) + baseline, 'k:')
		plt.xlabel(r"r ($\AA$)")
		plt.ylabel(r"G ($\AA^{-2}$)")
		plt.legend()

		plt.show()
		"""
		return energy

	def calc_num_forces(self,atoms):
		forces=np.zeros((len(atoms),3))
		pos0=atoms.positions.copy()
		con=constrained_indices(atoms)

		for i in range(len(atoms)):
			if i not in con:
				for j in range(3):
#					if i%self.rank==0:
					if i%self.size==self.rank:
#						print(i, self.rank)
						a=atoms.copy()
						pos=pos0.copy()
						pos[i][j]-=0.005
						a.positions=pos
						el=self.calc_energy(a)
						pos[i][j]+=2*0.005
						a.positions=pos
						er=self.calc_energy(a)
						forces[i][j]=(el-er)/0.01
						for k in range(self.size):
							req = self.comm.isend(forces[i][j], dest=k, tag=22+i*len(atoms)+j)
							req.wait()
		for i in range(len(atoms)):
			if i not in con:
				for j in range(3):
					if i%self.size!=self.rank:
#					else:
						req = self.comm.irecv(source=i%self.size, tag=22+i*len(atoms)+j)
						forces[i][j]=req.wait()
		return forces

#
	def calculate(self, atoms=None, properties=['energy'],system_changes=all_changes):
		Calculator.calculate(self, atoms, properties, system_changes)
		self.optparams=True
		if self.rank==0:
			self.optparams=True
			energy=self.calc_energy(atoms)
			print('Rw: ',(energy+100)/100)
			for i in range(1,self.size):
				req = self.comm.isend(energy, dest=i, tag=11)
				req.wait()
		else:
			energy=None
			req = self.comm.irecv(source=0, tag=11)
			energy = req.wait()
#		print('energy: ',energy, self.rank)
		if self.calcforces==True:
			self.optparams=False
			forces=self.calc_num_forces(atoms)
		else:
			forces=np.zeros((len(atoms),3))
		self.results['energy'], self.results['forces'] = energy, forces

	def print_pdf(self,atoms,show=False,label=None,printparams=False):
		if self.rank==0:
			rand=str(self.rank)
			structureFile = rand+'rdftempf.cif'
			write(structureFile,atoms,format='cif',parallel=False)
			dataFile = self.Gr
			xmi = self.pdfrange[0]
			xma = self.pdfrange[1]
			niPDF = PDFContribution("pdf")
			niPDF.loadData(dataFile)
			niPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
#			niStructure = loadStructure(structureFile,fmt='cif')
			niStructure = loadCrystal(structureFile)
			niPDF.addStructure("G", niStructure,periodic=self.periodic)
			if self.psize != None:
				from diffpy.srfit.pdf.characteristicfunctions import sphericalCF
				niPDF.registerFunction(sphericalCF, name = "f")
				niPDF.setEquation("f * pdf")
			niFit = FitRecipe()
			niFit.addContribution(niPDF)
			niFit.addVar(niPDF.scale, self.scale)
			niFit.addVar(niPDF.qdamp, self.qdamp, fixed=True)
			if self.psize != None:
				niFit.addVar(niPDF.psize, self.psize)
			niFit.clearFitHooks()
			leastsq(niFit.residual, niFit.values)
			niResults = FitResults(niFit)
			if printparams:
				print(niResults)
			res=str(niResults).strip().split()
			for i in range(len(res)):
        			if res[i]=='Rw':
                			rw=float(res[i+1])
                			break
			energy=rw
			print('Rw: '+str(rw))
			try:
				os.remove(structureFile)
			except:
				print("Can not delete the file as it doesn't exists")
#		if os.path.exists(structureFile):
#			os.remove(structureFile)
#		else:
#			print("Can not delete the file as it doesn't exists")
			r = niFit.pdf.profile.x
			gobs = niFit.pdf.profile.y

# Get the calculated PDF and compute the difference between the calculated and
# measured PDF
			gcalc = niFit.pdf.evaluate()
#			fr = niFit.G.evaluateEquation("f")
#			fr *= gobs.max()/fr[0]
			baseline = 1.1 * gobs.min()
			gdiff = gobs - gcalc

# Plot!
#		plt.figure()
			b=atoms.copy()
			try:
				fig, axarr = plt.subplots(1, 4, figsize=(14, 2.5),gridspec_kw={'width_ratios': [2, 1, 1, 1]})
				b=atoms.copy()
				pos=b.positions.copy()
				mins=[np.amin(pos[:,0]),np.amin(pos[:,1]),np.amin(pos[:,2])]
				pos[:,0]+=1.0-mins[0]
				pos[:,1]+=1.0-mins[1]
				pos[:,2]+=1.0-mins[2]
				maxs=[np.amax(pos[:,0]),np.amax(pos[:,1]),np.amax(pos[:,2])]
				tmax=np.amax(maxs)
				newcell=[[tmax+1.0,0.0,0.0],[0.0,tmax+1.0,0.0],[0.0,0.0,tmax+1.0]]
				b.cell=newcell
				b.positions=pos
				b.center
				plot_atoms(b, axarr[1],rotation=('0x,0y,0z'))
				axarr[1].set_axis_off()
				plot_atoms(b, axarr[2],rotation=('90x,0y,0z'))
				axarr[2].set_axis_off()
				plot_atoms(b, axarr[3],rotation=('90x,0y,90z'))
				axarr[3].set_axis_off()
			except:
				fig, axarr = plt.subplots(1, 2, figsize=(12, 6),gridspec_kw={'width_ratios': [999,1]})
				axarr[1].axis('off')
			axarr[0].plot(r, gobs, 'bo', label="G(r) data",
         			markerfacecolor='none', markeredgecolor='b')
			axarr[0].plot(r, gcalc, 'r-', label="G(r) fit")
			axarr[0].plot(r, gdiff + baseline, 'c-', label="G(r) diff")
			axarr[0].plot(r, np.zeros_like(r) + baseline, 'k:')
#			axarr[0].plot(r, fr,       'k--', label='Size-dampening')
			axarr[0].text(0.85*xma,(gobs.min()+gdiff.min()),'$R_w$='+str(rw)[0:5])
			axarr[0].set_xlabel(r"r ($\AA$)")
			axarr[0].set_ylabel(r"G ($\AA^{-2}$)")
			axarr[0].legend()
			if label!=None:
				plt.suptitle(label, fontsize=14)
#		plot_atoms(atoms, axarr[1], radii=0.99, rotation=('0x,0y,0z'))
# remove constraints
#		del atoms.constraints
			if show==False:
				if label==None:
					plt.savefig('PDFstructure.png')
				else:
					plt.savefig(label)
			else:
				plt.show()
		"""
		plt.figure()
		plt.plot(r, gobs, 'bo', label="G(r) data",
         		markerfacecolor='none', markeredgecolor='b')
		plt.plot(r, gcalc, 'r-', label="G(r) fit")
		plt.plot(r, gdiff + baseline, 'g-', label="G(r) diff")
		plt.plot(r, np.zeros_like(r) + baseline, 'k:')
		plt.text(0.85*xma,0.95*gobs.min()+baseline,'$R_w$='+str(rw)[0:5])
		plt.xlabel(r"r ($\AA$)")
		plt.ylabel(r"G ($\AA^{-2}$)")
		plt.legend()
		if show==False:
			plt.savefig('PDFstructure.png')
		else:
			plt.show()
		"""

class RDF2phase(Calculator):
	"""
	RDF
	"""
	implemented_properties = ['forces', 'energy']

	def __init__(self, struc2,Gr, pdfrange,delta=0.1,scale=0.04,qdamp=0.03,weightphase1=0.5,calcforces=False,periodic=True,label=None):
		Calculator.__init__(self)
		self.Gr = Gr
		self.struc2 = struc2
		self.weightphase1 = weightphase1
		self.pdfrange = pdfrange
		self.delta = delta
		self.calcforces=calcforces
		self.scale=scale
		self.qdamp=qdamp
		self.periodic=periodic
		self.comm = MPI.COMM_WORLD
		self.rank = self.comm.Get_rank()
		self.size = MPI.COMM_WORLD.Get_size()
		if label==None:
			self.label='rdftmpfile'
		else:
			self.label=label
#	def rmse(predictions, targets):
#		return np.sqrt(((predictions - targets) ** 2).mean())

	def calc_energy(self,atoms):
#		ff=io.StringIO()
#		rand=str(np.random.randint(10000))
		rand=str(self.rank)
#		print(rand)
		structureFile = self.label+rand+'.cif'
		write(structureFile,atoms,format='cif',parallel=False)
#		write(ff,atoms,format='cif')
		dataFile = self.Gr
#		structureFile = ff
		xmi = self.pdfrange[0]
		xma = self.pdfrange[1]
		"""
		niPDF = PDFContribution("G_ni")
		niPDF.loadData(dataFile)
		niPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
		niStructure = loadCrystal(structureFile)
#		niStructure = loadStructure(structureFile,fmt='cif')
		niPDF.addStructure("G_ni", niStructure,periodic=self.periodic)
		nibPDF = PDFContribution("G_si")
		nibPDF.loadData(dataFile)
		nibPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
		nibStructure = loadCrystal(self.struc2)
		nibPDF.addStructure("G_si", nibStructure,periodic=self.periodic)
		"""
		profile = Profile()
		# Load data and add it to the profile
		parser = PDFParser()
		parser.parseFile(dataFile)
		profile.loadParsedData(parser)
		profile.setCalculationRange(xmin = xmi, xmax = xma)
		niPDF = PDFGenerator("G_ni")
		niStructure = loadCrystal(structureFile)
		niPDF.setStructure(niStructure,periodic=self.periodic)
#		niPDF.addStructure("G_ni", niStructure,periodic=self.periodic)
#		stru = loadCrystal(niciffile)
#		generator_ni.setStructure(stru)
# The generator for the second phase. We call it "G_si".
		nibPDF = PDFGenerator("G_si")
		nibStructure = loadCrystal(self.struc2)
		nibPDF.setStructure( nibStructure,periodic=self.periodic)
#		stru = loadCrystal(siciffile)
#		generator_si.setStructure(stru)
####################################################################################
####################################################################################
		contribution = FitContribution("nisi")
		contribution.addProfileGenerator(niPDF)
		contribution.addProfileGenerator(nibPDF)
		contribution.setProfile(profile, xname = "r")
		contribution.setEquation("scale * (G_ni +  G_si)")
		# Make the FitRecipe and add the FitContribution.
		recipe = FitRecipe()
		recipe.addContribution(contribution)

		recipe.newVar("scale_ni", self.weightphase1)
		recipe.constrain(niPDF.scale, "scale_ni")
		recipe.constrain(nibPDF.scale, "1 - scale_ni")
		# We also want the resolution factor to be the same on each.
		recipe.newVar("qdamp", self.qdamp)
		recipe.constrain(niPDF.qdamp, "qdamp")
		recipe.constrain(nibPDF.qdamp, "qdamp")

		# Vary the gloabal scale as well.
		recipe.addVar(contribution.scale, self.scale)
		niFit = recipe
		"""
		niFit = FitRecipe()
		niFit.addContribution(niPDF)
#		niFit.addVar(niPDF.scale, self.scale*self.weightphase1)
#		niFit.addVar(niPDF.psize, 100)
#		niFit.addVar(niPDF.delta1,2)
		niFit.addVar(niPDF.qdamp, self.qdamp, fixed=True)
################## second phase########################
		nibPDF = PDFContribution("phase2")
		nibPDF.loadData(dataFile)
		nibPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
		nibStructure = loadCrystal(self.struc2)
		nibPDF.addStructure("phase2", nibStructure,periodic=self.periodic)
		niFit.addContribution(nibPDF)
#		niFit.addVar(nibPDF.scale, self.scale*(1-self.weightphase1))
#		niFit.newVar("qdamp_Phase2", self.qdamp)
#		niFit.constrain(nibPDF.qdamp, self.qdamp, "qdamp_Phase2")
		niFit.newVar("scale_Phase1", self.weightphase1)
		niFit.constrain(niPDF.scale, "scale_Phase1")
		niFit.constrain(nibPDF.scale, "1 - scale_Phase1")
		"""
################## second phase########################
		niFit.clearFitHooks()
		if self.optparams:
			leastsq(niFit.residual, niFit.values)
		niResults = FitResults(niFit)
		res=str(niResults).strip().split()
		for i in range(len(res)):
        		if res[i]=='Rw':
                		rw=float(res[i+1])
                		break
		energy=rw*100-100
#		try:
#			os.remove(structureFile)
#		except:
#			print("Can not delete the file as it doesn't exists")
#		if os.path.exists(structureFile):
#			os.remove(structureFile)
#		else:
#			print("Can not delete the file as it doesn't exists")
		"""
		r = niFit.nickel.profile.x
		gobs = niFit.nickel.profile.y

# Get the calculated PDF and compute the difference between the calculated and
# measured PDF
		gcalc = niFit.nickel.evaluate()
		baseline = 1.1 * gobs.min()
		gdiff = gobs - gcalc

# Plot!
		plt.figure()
		plt.plot(r, gobs, 'bo', label="G(r) data",
         		markerfacecolor='none', markeredgecolor='b')
		plt.plot(r, gcalc, 'r-', label="G(r) fit")
		plt.plot(r, gdiff + baseline, 'g-', label="G(r) diff")
		plt.plot(r, np.zeros_like(r) + baseline, 'k:')
		plt.xlabel(r"r ($\AA$)")
		plt.ylabel(r"G ($\AA^{-2}$)")
		plt.legend()

		plt.show()
		"""
		return energy

	def calc_num_forces(self,atoms):
		forces=np.zeros((len(atoms),3))
		pos0=atoms.positions.copy()
		con=constrained_indices(atoms)

		for i in range(len(atoms)):
			if i not in con:
				for j in range(3):
#					if i%self.rank==0:
					if i%self.size==self.rank:
#						print(i, self.rank)
						a=atoms.copy()
						pos=pos0.copy()
						pos[i][j]-=0.005
						a.positions=pos
						el=self.calc_energy(a)
						pos[i][j]+=2*0.005
						a.positions=pos
						er=self.calc_energy(a)
						forces[i][j]=(el-er)/0.01
						for k in range(self.size):
							req = self.comm.isend(forces[i][j], dest=k, tag=22+i*len(atoms)+j)
							req.wait()
		for i in range(len(atoms)):
			if i not in con:
				for j in range(3):
					if i%self.size!=self.rank:
#					else:
						req = self.comm.irecv(source=i%self.size, tag=22+i*len(atoms)+j)
						forces[i][j]=req.wait()
		return forces

#
	def calculate(self, atoms=None, properties=['energy'],system_changes=all_changes):
		Calculator.calculate(self, atoms, properties, system_changes)
		if self.rank==0:
			self.optparams=True
			energy=self.calc_energy(atoms)
			print('Rw: ',(energy+100)/100)
			for i in range(1,self.size):
				req = self.comm.isend(energy, dest=i, tag=11)
				req.wait()
		else:
			energy=None
			req = self.comm.irecv(source=0, tag=11)
			energy = req.wait()
#		print('energy: ',energy, self.rank)
		if self.calcforces==True:
			self.optparams=False
			forces=self.calc_num_forces(atoms)
		else:
			forces=np.zeros((len(atoms),3))
		self.results['energy'], self.results['forces'] = energy, forces


class RDFmultiphase(Calculator):
	"""
	RDF
	"""
	implemented_properties = ['forces', 'energy']

	def __init__(self, Gr, pdfrange,strucs=None,delta=0.1,scale=0.04,qdamp=0.03,weightphases=0.5,psize=None,delta2=None,delta1=None,biso=None,calcforces=False,periodic=True,plot=False,saveG=None,printparams=False,label=None):
		Calculator.__init__(self)
		self.Gr = Gr
		if strucs==None:
			self.struc2=[]
		elif isinstance(strucs, list):
			self.struc2 = strucs
		else:
			self.struc2 = [strucs]
		self.weightphase1 = weightphases
		self.pdfrange = pdfrange
		self.delta = delta
		self.delta2 = delta2
		self.delta1 = delta1
		self.biso = biso
		self.calcforces=calcforces
		self.scale=scale
		self.qdamp=qdamp
		self.psize=psize
		self.periodic=periodic
		self.plot=plot
		self.saveG=saveG
		self.comm = MPI.COMM_WORLD
		self.rank = self.comm.Get_rank()
		self.size = MPI.COMM_WORLD.Get_size()
		self.printparams=printparams
		if label==None:
			self.label='rdftmpfile'
		else:
			self.label=label
#	def rmse(predictions, targets):
#		return np.sqrt(((predictions - targets) ** 2).mean())

	def calc_energy(self,atoms):
#		ff=io.StringIO()
#		rand=str(np.random.randint(10000))
		rand=str(self.rank)
#		print(rand)
		structureFile = self.label+rand+'.cif'
		write(structureFile,atoms,format='cif',parallel=False)
#		write(ff,atoms,format='cif')
		dataFile = self.Gr
#		structureFile = ff
		xmi = self.pdfrange[0]
		xma = self.pdfrange[1]
		"""
		niPDF = PDFContribution("G_ni")
		niPDF.loadData(dataFile)
		niPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
		niStructure = loadCrystal(structureFile)
#		niStructure = loadStructure(structureFile,fmt='cif')
		niPDF.addStructure("G_ni", niStructure,periodic=self.periodic)
		nibPDF = PDFContribution("G_si")
		nibPDF.loadData(dataFile)
		nibPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
		nibStructure = loadCrystal(self.struc2)
		nibPDF.addStructure("G_si", nibStructure,periodic=self.periodic)
		"""
		profile = Profile()
		# Load data and add it to the profile
		parser = PDFParser()
		parser.parseFile(dataFile)
		profile.loadParsedData(parser)
		profile.setCalculationRange(xmin = xmi, xmax = xma)
		niPDF = PDFGenerator("G_ni")
		niStructure = loadCrystal(structureFile)
		niPDF.setStructure(niStructure,periodic=self.periodic)
#		niPDF.addStructure("G_ni", niStructure,periodic=self.periodic)
#		stru = loadCrystal(niciffile)
#		generator_ni.setStructure(stru)
# The generator for the second phase. We call it "G_si".
		nibPDF=[]
		for i in range(len(self.struc2)):
			nibPDF.append(PDFGenerator("G_"+str(i)))
			nibStructure = loadCrystal(self.struc2[i])
			nibPDF[i].setStructure( nibStructure,periodic=self.periodic)
#		stru = loadCrystal(siciffile)
#		generator_si.setStructure(stru)
####################################################################################
####################################################################################
		contribution = FitContribution("nisi")
		contribution.setProfile(profile, xname = "r")
		contribution.addProfileGenerator(niPDF)
		if self.psize != None:
			from diffpy.srfit.pdf.characteristicfunctions import sphericalCF
			contribution.registerFunction(sphericalCF, name = "f")
			pdfequation='scale * f * (G_ni '
		else:
			pdfequation='scale * (G_ni '
		for i in range(len(self.struc2)):
			contribution.addProfileGenerator(nibPDF[i])
			pdfequation+='+ G_'+str(i)+' '
		pdfequation+=')'
		contribution.setEquation(pdfequation)
		# Make the FitRecipe and add the FitContribution.
		recipe = FitRecipe()
		recipe.addContribution(contribution)

		if len(self.struc2)>=1:
			recipe.newVar("scale_ni", self.weightphase1)
			recipe.constrain(niPDF.scale, "scale_ni")
		for i in range(len(self.struc2)):
			recipe.constrain(nibPDF[i].scale, "(1 - scale_ni) / "+str(len(self.struc2)))
		# We also want the resolution factor to be the same on each.
		recipe.newVar("qdamp", self.qdamp)
#		recipe.constrain(niPDF.qdamp, "qdamp")
#		for i in range(len(self.struc2)):
#			recipe.constrain(nibPDF[i].qdamp, "qdamp")
		if self.delta2!=None:
			recipe.addVar(niPDF.delta2, self.delta2)
		if self.delta1!=None:
			recipe.addVar(niPDF.delta1, self.delta1)
#			recipe.constrain(niPDF.delta2, "delta2")
#			for i in range(len(self.struc2)):
#				recipe.constrain(nibPDF[i].qdamp, "qdamp")
		if self.biso!=None:
			for i in range(len(self.biso[0])):
				recipe.newVar('Biso'+self.biso[0][i], self.biso[1][i])

		# Vary the gloabal scale as well.
		recipe.addVar(contribution.scale, self.scale)
		if self.psize != None:
#			recipe.newVar("psize", self.psize)
			recipe.addVar(contribution.psize, self.psize)
		niFit = recipe
		"""
		niFit = FitRecipe()
		niFit.addContribution(niPDF)
#		niFit.addVar(niPDF.scale, self.scale*self.weightphase1)
#		niFit.addVar(niPDF.psize, 100)
#		niFit.addVar(niPDF.delta1,2)
		niFit.addVar(niPDF.qdamp, self.qdamp, fixed=True)
################## second phase########################
		nibPDF = PDFContribution("phase2")
		nibPDF.loadData(dataFile)
		nibPDF.setCalculationRange(xmin=xmi, xmax=xma, dx=self.delta)
		nibStructure = loadCrystal(self.struc2)
		nibPDF.addStructure("phase2", nibStructure,periodic=self.periodic)
		niFit.addContribution(nibPDF)
#		niFit.addVar(nibPDF.scale, self.scale*(1-self.weightphase1))
#		niFit.newVar("qdamp_Phase2", self.qdamp)
#		niFit.constrain(nibPDF.qdamp, self.qdamp, "qdamp_Phase2")
		niFit.newVar("scale_Phase1", self.weightphase1)
		niFit.constrain(niPDF.scale, "scale_Phase1")
		niFit.constrain(nibPDF.scale, "1 - scale_Phase1")
		"""
################## second phase########################
		niFit.clearFitHooks()
		if self.optparams:
			leastsq(niFit.residual, niFit.values)
		niResults = FitResults(niFit)
		res=str(niResults).strip().split()
		if self.printparams:
			print(niResults)
		if self.optparams==False:
			for i in range(len(res)):
        			if res[i]=='Rw':
                			rw=float(res[i+1])
                			break
		else:
			for i in range(len(res)):
        			if res[i]=='Rw':
                			rw=float(res[i+1])
        			elif res[i]=='scale':
                			self.scale=float(res[i+1])
        			elif res[i]=='qdamp':
                			self.qdamp=float(res[i+1])
        			elif res[i]=='psize':
                			self.psize=float(res[i+1])
        			elif res[i]=='delta2':
                			self.delta2=float(res[i+1])
        			elif res[i]=='delta1':
                			self.delta1=float(res[i+1])
        			if self.biso!=None:
        				for j in range(len(self.biso[0])):
        					if res[i]=='Biso'+self.biso[0][j]:
        						self.biso[1][j]=float(res[i+1])
#		print(rw,self.scale,self.qdamp,self.psize,self.delta2)
		energy=rw*100-100


		if self.plot==True:
			r = niFit.nisi.profile.x
			gobs = niFit.nisi.profile.y
			gcalc = niFit.nisi.evaluate()
#			gcalc = niFit.nisi.ycalc
			baseline = 1.1 * gobs.min()
			gdiff = gobs - gcalc
			plt.figure(figsize=(9, 6))
			plt.plot(r, gobs, 'bo', label=r"$G(r)_{observed}$",ms=3)
#			plt.plot(r, gcalc, 'r-', label=r"$G(r)_{calc}$",lw=3)
			plt.plot(r, gcalc, 'r-', label=r"$G(r)_{phase 1 + 3}$",lw=3)
			plt.plot(r, gdiff + baseline, 'g-', label=r"$G(r)_{obs}-G(r)_{phase 1 + 3}$",lw=3)
			plt.plot(r, np.zeros_like(r) + baseline, 'k:')
			plt.xlabel(r"r [$\AA$]")
			plt.ylabel(r"G [a.u.]")
			plt.legend()
			plt.rcParams.update({'font.size': 18})
			plt.xlim(self.pdfrange)
			plt.ylim([1.1*min(gdiff + baseline),1.6*max(gobs)])
			leg=plt.legend(ncol=1,markerscale=3,handlelength=1.5)
			for line in leg.get_lines():
    				line.set_linewidth(3.0)
			plt.savefig('PDFstructure.png',dpi=600,transparent=True)
		if self.saveG!=None:
			r = niFit.nisi.profile.x
			gobs = niFit.nisi.profile.y
			gcalc = niFit.nisi.evaluate()
			with open(self.saveG, 'w') as G_file:
				for i in range(len(r)):
					G_file.write(str(r[i])+' '+str(gobs[i])+' '+str(gcalc[i])+'\n')
#		try:
#			os.remove(structureFile)
#		except:
#			print("Can not delete the file as it doesn't exists")
#		if os.path.exists(structureFile):
#			os.remove(structureFile)
#		else:
#			print("Can not delete the file as it doesn't exists")
		"""
		r = niFit.nickel.profile.x
		gobs = niFit.nickel.profile.y

# Get the calculated PDF and compute the difference between the calculated and
# measured PDF
		gcalc = niFit.nickel.evaluate()
		baseline = 1.1 * gobs.min()
		gdiff = gobs - gcalc

# Plot!
		plt.figure()
		plt.plot(r, gobs, 'bo', label="G(r) data",
         		markerfacecolor='none', markeredgecolor='b')
		plt.plot(r, gcalc, 'r-', label="G(r) fit")
		plt.plot(r, gdiff + baseline, 'g-', label="G(r) diff")
		plt.plot(r, np.zeros_like(r) + baseline, 'k:')
		plt.xlabel(r"r ($\AA$)")
		plt.ylabel(r"G ($\AA^{-2}$)")
		plt.legend()

		plt.show()
		"""
		return energy

	def calc_num_forces(self,atoms):
		forces=np.zeros((len(atoms),3))
		pos0=atoms.positions.copy()
		con=constrained_indices(atoms)

		for i in range(len(atoms)):
			if i not in con:
				for j in range(3):
#					if i%self.rank==0:
					if i%self.size==self.rank:
#						print(i, self.rank)
						a=atoms.copy()
						pos=pos0.copy()
						pos[i][j]-=0.005
						a.positions=pos
						el=self.calc_energy(a)
						pos[i][j]+=2*0.005
						a.positions=pos
						er=self.calc_energy(a)
						forces[i][j]=(el-er)/0.01
						for k in range(self.size):
							req = self.comm.isend(forces[i][j], dest=k, tag=22+i*len(atoms)+j)
							req.wait()
		for i in range(len(atoms)):
			if i not in con:
				for j in range(3):
					if i%self.size!=self.rank:
#					else:
						req = self.comm.irecv(source=i%self.size, tag=22+i*len(atoms)+j)
						forces[i][j]=req.wait()
		return forces

#
	def calculate(self, atoms=None, properties=['energy'],system_changes=all_changes):
		Calculator.calculate(self, atoms, properties, system_changes)
		self.optparams=True
		if self.rank==0:
			energy=self.calc_energy(atoms)
			print('Rw: ',(energy+100)/100)
			for i in range(1,self.size):
				req = self.comm.isend(energy, dest=i, tag=11)
				req.wait()
				reqps = self.comm.isend(self.psize, dest=i, tag=12)
				reqps.wait()
				reqs = self.comm.isend(self.scale, dest=i, tag=13)
				reqs.wait()
				reqq = self.comm.isend(self.qdamp, dest=i, tag=14)
				reqq.wait()
				reqd = self.comm.isend(self.delta2, dest=i, tag=15)
				reqd.wait()
				reqb = self.comm.isend(self.biso, dest=i, tag=16)
				reqb.wait()
				reqd1 = self.comm.isend(self.delta1, dest=i, tag=17)
				reqd1.wait()
		else:
			energy=None
			req = self.comm.irecv(source=0, tag=11)
			energy = req.wait()
			reqps = self.comm.irecv(source=0, tag=12)
			self.psize = reqps.wait()
			reqs = self.comm.irecv(source=0, tag=13)
			self.scale = reqs.wait()
			reqq = self.comm.irecv(source=0, tag=14)
			self.qdamp = reqq.wait()
			reqd = self.comm.irecv(source=0, tag=15)
			self.delta2 = reqd.wait()
			reqb = self.comm.irecv(source=0, tag=16)
			self.biso = reqb.wait()
			reqd1 = self.comm.irecv(source=0, tag=17)
			self.delta1 = reqd1.wait()
#		print('energy: ',energy, self.rank)
		if self.calcforces==True:
			self.optparams=False
			forces=self.calc_num_forces(atoms)
		else:
			forces=np.zeros((len(atoms),3))
		self.results['energy'], self.results['forces'] = energy, forces
